@@ -3,9 +3,9 @@
 #include <dirent.h>
 #include <errno.h>
 #include <fcntl.h>
-#include <getopt.h>
 #include <glob.h>
 #include <grp.h>
+#include <popt.h>
 #include <pwd.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -22,18 +22,19 @@
 #define GLOB_ABORTED GLOB_ABEND
 #endif
 
-static char * defTabooExts[] = { ".rpmsave", ".rpmorig", "~", ",v" };
+static char * defTabooExts[] = { ".rpmsave", ".rpmorig", "~", ",v",
+				 ".rpmnew", ".swp" };
 static int defTabooCount = sizeof(defTabooExts) / sizeof(char *);
 
 /* I shouldn't use globals here :-( */
 static char ** tabooExts = NULL;
 int tabooCount = 0;
 
-static int readConfigFile(char * configFile, logInfo * defConfig, 
+static int readConfigFile(const char * configFile, logInfo * defConfig, 
 			  logInfo ** logsPtr, int * numLogsPtr);
 static int globerr(const char * pathname, int theerr);
 
-static int isolateValue(char * fileName, int lineNum, char * key, 
+static int isolateValue(const char * fileName, int lineNum, char * key, 
 			char ** startPtr, char ** endPtr) {
     char * chptr = *startPtr;
 
@@ -53,17 +54,14 @@ static int isolateValue(char * fileName, int lineNum, char * key,
 
     while (*chptr != '\n') chptr++;
 
-    while (isblank(*chptr)) chptr--;
+    while (isspace(*chptr)) chptr--;
 
-    if (*chptr == '\n')
-	*endPtr = chptr;
-    else
-	*endPtr = chptr + 1;
+    *endPtr = chptr + 1;
 
     return 0;
 }
 
-static char *readPath(char *configFile, int lineNum, char *key,
+static char *readPath(const char *configFile, int lineNum, char *key,
 		      char **startPtr) {
     char oldchar;
     char *endtag, *chptr;
@@ -96,7 +94,7 @@ static char *readPath(char *configFile, int lineNum, char *key,
 	return NULL;
 }
 
-static char * readAddress(char * configFile, int lineNum, char * key, 
+static char * readAddress(const char * configFile, int lineNum, char * key, 
 			  char ** startPtr) {
     char oldchar;
     char * endtag, * chptr;
@@ -127,7 +125,7 @@ static char * readAddress(char * configFile, int lineNum, char * key,
 	return NULL;
 }
 
-int readConfigPath(char * path, logInfo * defConfig, 
+int readConfigPath(const char * path, logInfo * defConfig, 
 			  logInfo ** logsPtr, int * numLogsPtr) {
     struct stat sb;
     DIR * dir;
@@ -223,7 +221,7 @@ static int globerr(const char * pathname, int theerr) {
     return 1;
 }
 
-static int readConfigFile(char * configFile, logInfo * defConfig, 
+static int readConfigFile(const char * configFile, logInfo * defConfig, 
 			  logInfo ** logsPtr, int * numLogsPtr) {
     int fd;
     char * buf, * endtag;
@@ -244,6 +242,8 @@ static int readConfigFile(char * configFile, logInfo * defConfig,
     int createMode;
     struct stat sb, sb2;
     glob_t globResult;
+    const char ** argv;
+    int argc, argNum;
 
     /* FIXME: createOwner and createGroup probably shouldn't be fixed
        length arrays -- of course, if we aren't run setuid it doesn't
@@ -352,6 +352,14 @@ static int readConfigFile(char * configFile, logInfo * defConfig,
 	    } else if (!strcmp(start, "nodelaycompress")) {
 		newlog->flags &= ~LOG_FLAG_DELAYCOMPRESS;
 
+		*endtag = oldchar, start = endtag;
+	    } else if (!strcmp(start, "sharedscripts")) {
+		newlog->flags |= LOG_FLAG_SHAREDSCRIPTS;
+
+		*endtag = oldchar, start = endtag;
+	    } else if (!strcmp(start, "nosharedscripts")) {
+		newlog->flags &= ~LOG_FLAG_SHAREDSCRIPTS;
+
 		*endtag = oldchar, start = endtag;
 	    } else if (!strcmp(start, "copytruncate")) {
 		newlog->flags |= LOG_FLAG_COPYTRUNCATE;
@@ -394,7 +402,7 @@ static int readConfigFile(char * configFile, logInfo * defConfig,
 				createOwner, createGroup, &foo);
 		if (rc == 4) {
 		    message(MESS_ERROR, "%s:%d extra arguments for "
-			    "create\n", configFile, lineNum, start[length]);
+			    "create\n", configFile, lineNum);
 		    return 1;
 		}
 
@@ -404,7 +412,7 @@ static int readConfigFile(char * configFile, logInfo * defConfig,
 		if (rc > 1) {
 		    pw = getpwnam(createOwner);
 		    if (!pw) {
-			message(MESS_ERROR, "%s:%d unkown user '%s'\n", 
+			message(MESS_ERROR, "%s:%d unknown user '%s'\n", 
 				configFile, lineNum, createOwner);
 			return 1;
 		    } 
@@ -414,7 +422,7 @@ static int readConfigFile(char * configFile, logInfo * defConfig,
 		if (rc > 2) {
 		    group = getgrnam(createGroup);
 		    if (!group) {
-			message(MESS_ERROR, "%s:%d unkown group '%s'\n", 
+			message(MESS_ERROR, "%s:%d unknown group '%s'\n", 
 				configFile, lineNum, createGroup);
 			return 1;
 		    } 
@@ -503,20 +511,16 @@ static int readConfigFile(char * configFile, logInfo * defConfig,
 		    oldchar = *endtag, *endtag = '\0';
 
 		    newlog->rotateCount = strtoul(start, &chptr, 0);
-		    if (*chptr) {
-			message(MESS_ERROR, "%s:%d bad rotation count'%s'\n",
+		    if (*chptr || newlog->rotateCount < 0) {
+			message(MESS_ERROR, "%s:%d bad rotation count '%s'\n",
 				    configFile, lineNum, start);
 			return 1;
 		    }
-
 		    *endtag = oldchar, start = endtag;
 		}
 	    } else if (!strcmp(start, "errors")) {
-		*endtag = oldchar, start = endtag;
-		if (!(newlog->errAddress = readAddress(configFile, lineNum, 
-						       "error", &start))) {
-		    return 1;
-		}
+		message(MESS_DEBUG, "%s: %d: the errors directive is deprecated and no longer used.\n",
+			configFile, lineNum);
 	    } else if (!strcmp(start, "mail")) {
 		*endtag = oldchar, start = endtag;
 		if (!(newlog->logAddress = readAddress(configFile, lineNum, 
@@ -552,8 +556,8 @@ static int readConfigFile(char * configFile, logInfo * defConfig,
 		while (*start != '\n') start++;
 	    } else if (!strcmp(start, "tabooext")) {
 		if (newlog != defConfig) {
-		    message(MESS_ERROR, "%s:%d include may not appear inside "
-			    "of log file definition", configFile, lineNum);
+		    message(MESS_ERROR, "%s:%d tabooext may not appear inside "
+			    "of log file definition\n", configFile, lineNum);
 		    return 1;
 		}
 
@@ -594,7 +598,7 @@ static int readConfigFile(char * configFile, logInfo * defConfig,
 	    } else if (!strcmp(start, "include")) {
 		if (newlog != defConfig) {
 		    message(MESS_ERROR, "%s:%d include may not appear inside "
-			    "of log file definition", configFile, lineNum);
+			    "of log file definition\n", configFile, lineNum);
 		    return 1;
 		}
 
@@ -627,8 +631,8 @@ static int readConfigFile(char * configFile, logInfo * defConfig,
 
 		if (!S_ISDIR(sb.st_mode)) {
 		    message(MESS_ERROR, "%s:%d olddir path %s is not a "
-				"directory %s: %s\n", configFile, lineNum, 
-				newlog->oldDir, strerror(errno));
+				"directory\n", configFile, lineNum, 
+				newlog->oldDir);
 		    return 1;
 		}
 
@@ -646,6 +650,50 @@ static int readConfigFile(char * configFile, logInfo * defConfig,
 		}
 
 		message(MESS_DEBUG, "extension is now %s\n", newlog->extension);
+
+	    } else if (!strcmp(start, "compresscmd")) {
+		*endtag = oldchar, start = endtag;
+		if (!(newlog->compress_prog = readPath(configFile, lineNum, "compress", &start))) {
+		    return 1;
+		}
+
+		if (access(newlog->compress_prog, X_OK)) {
+		    message(MESS_ERROR, "%s:%d compression program %s is not an executable file\n", configFile, lineNum, 
+				newlog->compress_prog);
+		    return 1;
+		}
+
+		message(MESS_DEBUG, "compress_prog is now %s\n", newlog->compress_prog);
+
+	    } else if (!strcmp(start, "uncompresscmd")) {
+		*endtag = oldchar, start = endtag;
+		if (!(newlog->uncompress_prog = readPath(configFile, lineNum, "uncompress", &start))) {
+		    return 1;
+		}
+
+		if (access(newlog->uncompress_prog, X_OK)) {
+		    message(MESS_ERROR, "%s:%d uncompression program %s is not an executable file\n", configFile, lineNum, 
+				newlog->uncompress_prog);
+		    return 1;
+		}
+
+		message(MESS_DEBUG, "uncompress_prog is now %s\n", newlog->uncompress_prog);
+
+	    } else if (!strcmp(start, "compressoptions")) {
+		*endtag = oldchar, start = endtag;
+		if (!(newlog->compress_options = readPath(configFile, lineNum, "compressoptions", &start))) {
+		    return 1;
+		}
+
+		message(MESS_DEBUG, "compress_options is now %s\n", newlog->compress_options);
+
+	    } else if (!strcmp(start, "compressext")) {
+		*endtag = oldchar, start = endtag;
+		if (!(newlog->compress_ext = readPath(configFile, lineNum, "compress-ext", &start))) {
+		    return 1;
+		}
+
+		message(MESS_DEBUG, "compress_ext is now %s\n", newlog->compress_ext);
 	    } else {
 		message(MESS_ERROR, "%s:%d unknown option '%s' "
 			    "-- ignoring line\n", configFile, lineNum, start);
@@ -663,7 +711,7 @@ static int readConfigFile(char * configFile, logInfo * defConfig,
 
 	    lineNum++;
 	    start++;
-	} else if (*start == '/' || *start == '"') {
+	} else if (*start == '/' || *start == '"' || *start == '\'') {
 	    if (newlog != defConfig) {
 		message(MESS_ERROR, "%s:%d unexpected log filename\n", 
 			configFile, lineNum);
@@ -675,79 +723,70 @@ static int readConfigFile(char * configFile, logInfo * defConfig,
 	    newlog = *logsPtr + *numLogsPtr - 1;
 	    memcpy(newlog, defConfig, sizeof(*newlog));
 
-	    if (*start == '"') {
-		start++;
-		endtag = strchr(start, '"');
-		if (!endtag || (strchr(start, '\n') < endtag)) {
-		    message(MESS_ERROR, "%s:%d end \" expected",
-			    configFile, lineNum);
-		}
-	    } else {
-		endtag = start;
-		while (!isspace(*endtag) && *endtag != '{') endtag++;
+	    endtag = start;
+	    while (*endtag != '{' && *endtag != '\0') endtag++;
+	    if (*endtag != '{') {
+		message(MESS_ERROR, "%s:%d missing end of line\n",
+			configFile, lineNum);
 	    }
-
-	    oldchar = *endtag;
 	    *endtag = '\0';
 
-	    rc = glob(start, GLOB_NOCHECK, globerr, &globResult);
-	    if (rc == GLOB_ABORTED) return 1;
+	    if (poptParseArgvString(start, &argc, &argv)) {
+		message(MESS_ERROR, "%s:%d error parsing filename\n",
+			configFile, lineNum);
+		return 1;
+	    } else if (argc < 1) {
+		message(MESS_ERROR, "%s:%d { expected after log file name(s)\n",
+			configFile, lineNum);
+		return 1;
+	    }
 
-	    newlog->files = malloc(sizeof(*newlog->files) * 
-				   globResult.gl_pathc);
+	    /* XXX this leaks the result of the glob <shrug> */
+	    newlog->files = NULL;
 	    newlog->numFiles = 0;
+	    for (argNum = 0; argNum < argc; argNum++) {
+		rc = glob(argv[argNum], GLOB_NOCHECK, globerr, &globResult);
+		if (rc == GLOB_ABORTED) {
+		    message(MESS_ERROR, "%s:%d glob failed for %s\n",
+			    configFile, lineNum, argv[argNum]);
+		    return 1;
+		}
+
+		newlog->files = realloc(newlog->files, sizeof(*newlog->files) * 
+				   (newlog->numFiles + globResult.gl_pathc));
 
-	    for (i = 0; i < globResult.gl_pathc; i++) {
-		/* if we glob directories we can get false matches */
-		if (!lstat(globResult.gl_pathv[i], &sb) && S_ISDIR(sb.st_mode)) 
-		    continue;
+		for (i = 0; i < globResult.gl_pathc; i++) {
+		    /* if we glob directories we can get false matches */
+		    if (!lstat(globResult.gl_pathv[i], &sb) && 
+				    S_ISDIR(sb.st_mode)) 
+			continue;
 
-		for (j = 0; j < *numLogsPtr - 1; j++) {
-		    for (k = 0; k < (*logsPtr)[j].numFiles; k++) {
-			if (!strcmp((*logsPtr)[j].files[k], 
-				    globResult.gl_pathv[i])) {
-			    message(MESS_ERROR, 
-				    "%s:%d duplicate log entry for %s\n",
-				    configFile, lineNum, 
-				    globResult.gl_pathv[i]);
-			    return 1;
+		    for (j = 0; j < *numLogsPtr - 1; j++) {
+			for (k = 0; k < (*logsPtr)[j].numFiles; k++) {
+			    if (!strcmp((*logsPtr)[j].files[k], 
+					globResult.gl_pathv[i])) {
+				message(MESS_ERROR, 
+					"%s:%d duplicate log entry for %s\n",
+					configFile, lineNum, 
+					globResult.gl_pathv[i]);
+				return 1;
+			    }
 			}
 		    }
+
+		    newlog->files[newlog->numFiles] = 
+			    globResult.gl_pathv[i];
+		    newlog->numFiles++;
 		}
-
-		newlog->files[newlog->numFiles] = 
-			globResult.gl_pathv[i];
-		newlog->numFiles++;
 	    }
 
-	    newlog->globMem = globResult;
 	    newlog->pattern = strdup(start);
 
 	    message(MESS_DEBUG, "reading config info for %s\n", start);
 
-	    *endtag = oldchar;
-	    if (*endtag == '"') endtag++;
-	    start = endtag;
+	    free(argv);
 
-	    while (*start && isspace(*start) && *start != '{') {
-		if (*start == '\n') lineNum++;
-		start++;
-	    }
-
-	    if (*start != '{') {
-		message(MESS_ERROR, "%s:%d { expected after log file name\n",
-			configFile, lineNum);
-		return 1;
-	    }
-
-	    start++;
-	    while (isblank(*start) && *start != '\n') start++;
-	
-	    if (*start != '\n' && *start != '}') {
-		message(MESS_ERROR, "%s:%d unexpected text after {\n",
-			configFile, lineNum);
-		return 1;
-	    }
+	    start = endtag + 1;
 	} else if (*start == '}') {
 	    if (newlog == defConfig) {
 		message(MESS_ERROR, "%s:%d unxpected }\n", configFile, lineNum);
