@@ -3,7 +3,6 @@
 #include <dirent.h>
 #include <errno.h>
 #include <fcntl.h>
-#include <getopt.h>
 #include <popt.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -28,7 +27,7 @@ typedef struct {
 int debug = 0;
 char * mailCommand = DEFAULT_MAIL_COMMAND;
 
-static logState * findState(char * fn, logState ** statesPtr, 
+static logState * findState(const char * fn, logState ** statesPtr, 
 			    int * numStatesPtr) {
     int i;
     logState * states = *statesPtr;
@@ -63,10 +62,9 @@ static logState * findState(char * fn, logState ** statesPtr,
 
 static int runScript(char * logfn, char * script) {
     int fd;
-    char filespec[20];
+    char filespec[32];
     char * cmd;
     int rc;
-    struct stat sb;
 
     if (debug) {
 	message(MESS_DEBUG, "running script with arg %s: \"%s\"\n", 
@@ -74,25 +72,11 @@ static int runScript(char * logfn, char * script) {
 	return 0;
     }
 
-    strcpy(filespec, "/tmp/logrotXXXXXX");
-
-    mktemp(filespec);
-    if ((fd = open(filespec, O_CREAT | O_EXCL | O_WRONLY, 0700)) < 0) {
+    strcpy(filespec, "/tmp/logrotate.XXXXXX");
+    if ((fd = mkstemp(filespec)) < 0 || fchmod(fd, 0700)) {
 	message(MESS_DEBUG, "error creating %s: %s\n", filespec,
 		strerror(errno));
-	return -1;
-    }
-
-    /* make sure we didn't just follow a symlink as part of a DOS */
-    if (stat(filespec, &sb)) {
-	/* huh? */
-	close(fd);
-	return -1;
-    }
-
-    if (S_ISLNK(sb.st_mode)) {
-	message(MESS_DEBUG, "%s is a symlink! probable DOS attack!!\n");
-	close(fd);
+	if (fd >= 0) close(fd);
 	return -1;
     }
 
@@ -110,7 +94,7 @@ static int runScript(char * logfn, char * script) {
     close(fd);
 
     cmd = alloca(strlen(filespec) + strlen(logfn) + 20);
-    sprintf(cmd, "/bin/sh %s %s", filespec, logfn);
+    sprintf(cmd, "/bin/sh %s '%s'", filespec, logfn);
     rc = system(cmd);
 
     unlink(filespec);
@@ -217,7 +201,7 @@ int rotateSingleLog(logInfo * log, int logNum, logState ** statesPtr,
 
     message(MESS_DEBUG, "rotating file %s\n", log->files[logNum]);
 
-    if (log->flags & LOG_FLAG_COMPRESS) compext = COMPRESS_EXT;
+    if (log->flags & LOG_FLAG_COMPRESS) compext = log->compress_ext;
     
     if (stat(log->files[logNum], &sb)) {
 	if ((log->flags & LOG_FLAG_MISSINGOK) && (errno == ENOENT)) {
@@ -239,6 +223,18 @@ int rotateSingleLog(logInfo * log, int logNum, logState ** statesPtr,
 	else if (log->criterium == ROT_FORCE) {
 	    /* user forced rotation of logs from command line */
 	    doRotate = 1;
+	} else if (state->lastRotated.tm_year > now.tm_year || 
+		      (state->lastRotated.tm_year == now.tm_year && 
+			  (state->lastRotated.tm_mon > now.tm_mon ||
+			      (state->lastRotated.tm_mon == now.tm_mon &&
+			       state->lastRotated.tm_mday > now.tm_mday)
+			  )
+		      )
+		  ) {
+	    message(MESS_ERROR,
+	    "file %s last rotated in the future -- rotation forced\n",
+	    log->files[logNum]);
+	    doRotate = 1;
 	} else if (state->lastRotated.tm_year != now.tm_year || 
 		   state->lastRotated.tm_mon != now.tm_mon ||
 		   state->lastRotated.tm_mday != now.tm_mday) {
@@ -296,13 +292,16 @@ int rotateSingleLog(logInfo * log, int logNum, logState ** statesPtr,
 	disposeName = alloca(strlen(dirName) + strlen(baseName) + 
 			    strlen(log->files[logNum]) + 10);
 
+
+	if (log->extension)
+	    fileext = log->extension;
+
 	if (log->extension &&
 		strncmp(&baseName[strlen(baseName)-strlen(log->extension)],
 		log->extension, strlen(log->extension)) == 0) {
 	    char *tempstr;
-	    fileext = log->extension;
-	    tempstr = calloc(strlen(baseName)-strlen(log->extension), sizeof(char));
-	    strncpy(tempstr, baseName,
+	    tempstr = calloc(strlen(baseName)-strlen(log->extension)+1, sizeof(char));
+	    strncat(tempstr, baseName,
 		    strlen(baseName)-strlen(log->extension));
 	    free(baseName);
 	    baseName = tempstr;
@@ -321,8 +320,8 @@ int rotateSingleLog(logInfo * log, int logNum, logState ** statesPtr,
 		char * command;
 
 		command = alloca(strlen(oldName) +
-				    strlen(COMPRESS_COMMAND) + 20);
-		sprintf(command, "%s %s", COMPRESS_COMMAND, oldName);
+				    strlen(log->compress_prog) + 1 + strlen(log->compress_options) + 20);
+		sprintf(command, "%s %s '%s'", log->compress_prog, log->compress_options, oldName);
 		message(MESS_DEBUG, "compressing previous log with: %s\n",
 				    command);
 		if (!debug && system(command)) {
@@ -380,7 +379,7 @@ int rotateSingleLog(logInfo * log, int logNum, logState ** statesPtr,
 	free(baseName);
 
 	if (!hasErrors) {
-	    if (log->pre) {
+	    if (log->pre && !(log->flags & LOG_FLAG_SHAREDSCRIPTS)) {
 		message(MESS_DEBUG, "running prerotate script\n");
 		if (runScript(log->files[logNum], log->pre)) {
 		    fprintf(errorFile, "error running prerotate script -- 
@@ -450,7 +449,8 @@ int rotateSingleLog(logInfo * log, int logNum, logState ** statesPtr,
 
 	    }
 
-	    if (!hasErrors && log->post) {
+	    if (!hasErrors && log->post && 
+		    !(log->flags & LOG_FLAG_SHAREDSCRIPTS)) {
 		message(MESS_DEBUG, "running postrotate script\n");
 		if (runScript(log->files[logNum], log->post)) {
 		    fprintf(errorFile, "error running postrotate script\n");
@@ -472,10 +472,9 @@ int rotateSingleLog(logInfo * log, int logNum, logState ** statesPtr,
 			!(log->flags & LOG_FLAG_DELAYCOMPRESS)) {
 		char * command;
 
-		command = alloca(strlen(finalName) + strlen(COMPRESS_COMMAND)
-				 + 20);
+		command = alloca(strlen(finalName) + strlen(log->compress_prog) + 1 + strlen(log->compress_options) + 20);
 
-		sprintf(command, "%s %s", COMPRESS_COMMAND, finalName);
+		sprintf(command, "%s %s '%s'", log->compress_prog, log->compress_options, finalName);
 		message(MESS_DEBUG, "compressing new log with: %s\n", command);
 		if (!debug && system(command)) {
 		    fprintf(errorFile, "failed to compress log %s\n", 
@@ -495,11 +494,13 @@ int rotateSingleLog(logInfo * log, int logNum, logState ** statesPtr,
 
 		if (mailFilename) {
 		    command = alloca(strlen(mailFilename) + 100 + 
-				     strlen(UNCOMPRESS_PIPE));
+				     strlen(log->uncompress_prog));
 
-		    if (log->flags & LOG_FLAG_COMPRESS)
+		    if ((log->flags & LOG_FLAG_COMPRESS) &&
+			    !(log->flags & LOG_FLAG_DELAYCOMPRESS) &&
+			    (log->flags & LOG_FLAG_MAILFIRST))
 			sprintf(command, "%s < %s | %s '%s' %s", 
-				    UNCOMPRESS_PIPE, mailFilename, mailCommand,
+				    log->uncompress_prog, mailFilename, mailCommand,
 				    log->files[logNum],
 				    log->logAddress);
 		    else
@@ -538,9 +539,6 @@ int rotateSingleLog(logInfo * log, int logNum, logState ** statesPtr,
 
 int rotateLogSet(logInfo * log, logState ** statesPtr, int * numStatesPtr, 
 		 int force) {
-    FILE * errorFile;
-    char * errorFileName = NULL;
-    int oldstderr = 0, newerr;
     int i;
     int hasErrors = 0;
 
@@ -585,62 +583,26 @@ int rotateLogSet(logInfo * log, logState ** statesPtr, int * numStatesPtr,
 	message(MESS_DEBUG, "old logs are removed\n");
     }
 
-    if (log->errAddress) {
-	message(MESS_DEBUG, "errors will be mailed to %s\n", log->errAddress);
-	errorFileName = strdup(tmpnam(NULL));
-
-	newerr = open(errorFileName, O_WRONLY | O_CREAT | O_EXCL, 0600);
-
-	if (newerr < 0) {
-	    message(MESS_ERROR, "error creating temporary file %s\n",
-			errorFileName);
-	    return 1;
+    if (log->pre && (log->flags & LOG_FLAG_SHAREDSCRIPTS)) {
+	message(MESS_DEBUG, "running shared prerotate script\n");
+	if (runScript(log->pattern, log->pre)) {
+	    fprintf(stderr, "error running shared prerotate script for %s-- 
+			leaving old logs in place\n", log->pattern);
+	    hasErrors = 1;
 	}
-	
-	if (!debug) {
-	    oldstderr = dup(2);
-	    dup2(newerr, 2);
-	    close(newerr);
-
-	    errorFile = fdopen(2, "w");
-
-	    setlinebuf(errorFile);
-
-	    fprintf(errorFile, "errors occured while rotating %s\n\n",
-		    log->pattern);
-	} else {
-	    errorFile = stderr;
-	}
-    } else {
-	message(MESS_DEBUG, "errors displayed on stderr\n");
-	errorFile = stderr;
     }
 
     for (i = 0; i < log->numFiles; i++)
 	hasErrors |= rotateSingleLog(log, i, statesPtr, numStatesPtr, 
-					errorFile);
+					stderr);
 
-    if (log->errAddress && !debug) {
-	fclose(errorFile);
-
-	close(2);
-	dup2(oldstderr, 2);
-	close(oldstderr);
-
-	if (hasErrors) {
-	    char * command;
-
-	    command = alloca(strlen(errorFileName) + strlen(log->errAddress) +
-				50);
-	    sprintf(command, "%s 'errors rotating logs' %s < %s\n",
-		    mailCommand, log->errAddress, errorFileName);
-	    if (system(command)) {
-		message(MESS_ERROR, "error mailing error log to %s -- errors "
-			"left in %s\n", log->errAddress, errorFileName);
-	    }
+    if (log->post && (log->flags & LOG_FLAG_SHAREDSCRIPTS)) {
+	message(MESS_DEBUG, "running shared postrotate script\n");
+	if (runScript(log->pattern, log->post)) {
+	    fprintf(stderr, 
+	       "error running shared postrotate script for %s\n", log->pattern);
+	    hasErrors = 1;
 	}
-
-	unlink(errorFileName);
     }
 
     return hasErrors;
@@ -649,6 +611,7 @@ int rotateLogSet(logInfo * log, logState ** statesPtr, int * numStatesPtr,
 static int writeState(char * stateFilename, logState * states, 
 		      int numStates) {
     FILE * f;
+    char * chptr;
     int i;
 
     f = fopen(stateFilename, "w");
@@ -658,10 +621,23 @@ static int writeState(char * stateFilename, logState * states,
 	return 1;
     }
 
-    fprintf(f, "logrotate state -- version 1\n");
+    fprintf(f, "logrotate state -- version 2\n");
 
     for (i = 0; i < numStates; i++) {
-	fprintf(f, "%s %d-%d-%d\n", states[i].fn, 
+	fputc('"', f);
+	for (chptr = states[i].fn; *chptr; chptr++) {
+	    switch (*chptr) {
+	      case '\\':
+	      case '\'':
+	      case '"':
+	          fputc('\\', f);
+	    }
+
+	    fputc(*chptr, f);
+	}
+
+	fputc('"', f);
+	fprintf(f, " %d-%d-%d\n", 
 		states[i].lastRotated.tm_year + 1900,
 		states[i].lastRotated.tm_mon + 1,
 		states[i].lastRotated.tm_mday);
@@ -676,7 +652,8 @@ static int readState(char * stateFilename, logState ** statesPtr,
 		     int * numStatesPtr) {
     FILE * f;
     char buf[1024];
-    char buf2[1024];
+    const char ** argv;
+    int argc;
     int year, month, day;
     int i;
     int line = 0;
@@ -694,7 +671,7 @@ static int readState(char * stateFilename, logState ** statesPtr,
 			stateFilename, strerror(errno));
 	    return 1;
 	}
-	fprintf(f, "logrotate state -- version 1\n");
+	fprintf(f, "logrotate state -- version 2\n");
 	fclose(f);
 	return 0;
     } else if (!f) {
@@ -709,7 +686,8 @@ static int readState(char * stateFilename, logState ** statesPtr,
 	return 1;
     }
 
-    if (strcmp(buf, "logrotate state -- version 1\n")) {
+    if (strcmp(buf, "logrotate state -- version 1\n") &&
+           strcmp(buf, "logrotate state -- version 2\n")) {
 	fclose(f);
 	message(MESS_ERROR, "bad top line in state file %s\n", stateFilename);
 	return 1;
@@ -727,9 +705,12 @@ static int readState(char * stateFilename, logState ** statesPtr,
 	    return 1;
 	}
 
+	buf[i - 1] = '\0';
+
 	if (i == 1) continue;
 
-	if (sscanf(buf, "%s %d-%d-%d\n", buf2, &year, &month, &day) != 4) {
+	if (poptParseArgvString(buf, &argc, &argv) || (argc != 2) ||
+		(sscanf(argv[1], "%d-%d-%d", &year, &month, &day) != 3)) {
 	    message(MESS_ERROR, "bad line %d in state file %s\n", 
 		    line, stateFilename);
 	    fclose(f);
@@ -739,14 +720,14 @@ static int readState(char * stateFilename, logState ** statesPtr,
 	/* Hack to hide earlier bug */
 	if ((year != 1900) && (year < 1996 || year > 2100)) {
 	    message(MESS_ERROR, "bad year %d for file %s in state file %s\n",
-			year, buf2, stateFilename);
+			year, argv[0], stateFilename);
 	    fclose(f);
 	    return 1;
 	}
 
 	if (month < 1 || month > 12) {
 	    message(MESS_ERROR, "bad month %d for file %s in state file %s\n",
-			month, buf2, stateFilename);
+			month, argv[0], stateFilename);
 	    fclose(f);
 	    return 1;
 	}
@@ -754,14 +735,14 @@ static int readState(char * stateFilename, logState ** statesPtr,
 	/* 0 to hide earlier bug */
 	if (day < 0 || day > 31) {
 	    message(MESS_ERROR, "bad day %d for file %s in state file %s\n",
-			day, buf2, stateFilename);
+			day, argv[0], stateFilename);
 	    fclose(f);
 	    return 1;
 	}
 
 	year -= 1900, month -= 1;
 
-	st = findState(buf2, statesPtr, numStatesPtr);
+	st = findState(argv[0], statesPtr, numStatesPtr);
 
 	st->lastRotated.tm_mon = month;
 	st->lastRotated.tm_mday = day;
@@ -770,6 +751,8 @@ static int readState(char * stateFilename, logState ** statesPtr,
 	/* fill in the rest of the st->lastRotated fields */
 	lr_time = mktime(&st->lastRotated);
 	st->lastRotated = *localtime(&lr_time);
+
+	free(argv);
     }
 
     fclose(f);
@@ -778,11 +761,14 @@ static int readState(char * stateFilename, logState ** statesPtr,
 
 }
 
-int main(int argc, char ** argv) {
+int main(int argc, const char ** argv) {
     logInfo defConfig = { NULL, NULL, 0, NULL, ROT_SIZE, 
 			  /* threshHold */ 1024 * 1024, 0,
-			  /* pre */ NULL, NULL, NULL, NULL, 
+			  /* pre, post */ NULL, NULL,
+			  /* logAddress */ NULL, 
 			  /* extension */ NULL, 
+			  /* compression */ COMPRESS_COMMAND,
+			  UNCOMPRESS_COMMAND, COMPRESS_OPTIONS, COMPRESS_EXT,
 			  /* flags */ LOG_FLAG_IFEMPTY,
 			  /* createMode */ NO_MODE, NO_UID, NO_GID };
     int numLogs = 0, numStates = 0;
@@ -793,14 +779,14 @@ int main(int argc, char ** argv) {
     int i;
     int rc = 0;
     int arg;
-    char ** files, ** file;
+    const char ** files, ** file;
     poptContext optCon;
     struct poptOption options[] = {
 	{ "debug", 'd', 0, 0, 'd', 
 		"Don't do anything, just test (implies -v)" },
 	{ "force", 'f', 0 , &force, 0, "Force file rotation" },
 	{ "mail", 'm', POPT_ARG_STRING, &mailCommand, 0, 
-		"Command to use to rotate logs", "command" },
+		"Command to use to mail logs", "command" },
 	{ "state", 's', POPT_ARG_STRING, &stateFile, 0, "Path of state file",
 		"statefile" },
 	{ "verbose", 'v', 0, 0, 'v', "Display messages during rotation" },
@@ -835,7 +821,7 @@ int main(int argc, char ** argv) {
     files = poptGetArgs(optCon);
     if (!files) {
 	fprintf(stderr, "logrotate " VERSION 
-		    " - Copyright (C) 1995 - Red Hat Software\n");
+		    " - Copyright (C) 1995-2001 Red Hat, Inc.\n");
 	fprintf(stderr, "This may be freely redistributed under the terms of "
 		    "the GNU Public License\n\n");
 	poptPrintUsage(optCon, stderr, 0);
