@@ -10,11 +10,14 @@
 #define LOG_FLAG_DELAYCOMPRESS	(1 << 3)
 #define LOG_FLAG_COPYTRUNCATE	(1 << 4)
 #define LOG_FLAG_MISSINGOK	(1 << 5)
+#define LOG_FLAG_MAILFIRST	(1 << 6)
+#define LOG_FLAG_SHAREDSCRIPTS	(1 << 7)
 
-#define MAIL_COMMAND "/bin/mail -s"
-#define COMPRESS_COMMAND "gzip -9"
+#define DEFAULT_MAIL_COMMAND "/bin/mail -s"
+#define COMPRESS_COMMAND "/bin/gzip"
+#define COMPRESS_OPTIONS "-9"
 #define COMPRESS_EXT ".gz"
-#define UNCOMPRESS_PIPE "gunzip"
+#define UNCOMPRESS_COMMAND "/bin/gunzip"
 
 #define NO_FORCE_ROTATE 0
 #define FORCE_ROTATE    1
@@ -31,16 +34,18 @@ typedef struct {
     int rotateCount;
     char * pre, * post;
     char * logAddress;
-    char * errAddress;
     char * extension;
+    char * compress_prog;
+    char * uncompress_prog;
+    char * compress_options;
+    char * compress_ext;
     int flags;
     mode_t createMode;		/* if any/all of these are -1, we use the */
     uid_t createUid;		/* attributes from the log file just rotated */
     gid_t createGid;
-    glob_t globMem;		/* at least we could theoretically free this */
 } logInfo;
 
-int readConfigPath(char * path, logInfo * defConfig, 
+int readConfigPath(const char * path, logInfo * defConfig, 
 			  logInfo ** logsPtr, int * numLogsPtr);
 
 extern int debug;
